/*****************************************************************************
 * XMClib v2.2.0 - XMC Peripheral Driver Library
 *
 * Copyright (c) 2015-2020, Infineon Technologies AG
 * All rights reserved.
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *
 * To improve the quality of the software, users are encouraged to share
 * modifications, enhancements or bug fixes with Infineon Technologies AG
 * at XMCSupport@infineon.com.
 ****************************************************************************/

/**
 * @file
 * @date 18 September,2018
 * @version 1.0.0
 *
 * History
 *
 * Version 1.0.0
 * - Initial
 *
 */

#include "xmc_gpio.h"
#include "xmc_dma.h"
#include "xmc_uart.h"

// 1ms task which reads from the ring buffer and copies to the working buffer
#define TICKS_PER_SECOND 1000

// Ring buffer using DMA
// The ring buffer is divided into two equal sized buffers
#define RING_BUFFER_SIZE 32
static volatile uint8_t ring_buffer[RING_BUFFER_SIZE];

static const XMC_DMA_CH_CONFIG_t dma_ch_config =
{
  {
	.enable_interrupt = true,
	.dst_transfer_width = XMC_DMA_CH_TRANSFER_WIDTH_8,
	.src_transfer_width = XMC_DMA_CH_TRANSFER_WIDTH_8,
	.dst_address_count_mode = XMC_DMA_CH_ADDRESS_COUNT_MODE_INCREMENT,
	.src_address_count_mode = XMC_DMA_CH_ADDRESS_COUNT_MODE_NO_CHANGE,
	.dst_burst_length = XMC_DMA_CH_BURST_LENGTH_1,
	.src_burst_length = XMC_DMA_CH_BURST_LENGTH_1,
	.transfer_flow = XMC_DMA_CH_TRANSFER_FLOW_P2M_DMA,
  },
  .block_size = RING_BUFFER_SIZE,
  .src_addr = (uint32_t)&(XMC_UART0_CH0->RBUF),
  .dst_addr = (uint32_t)&ring_buffer[0],
  .transfer_type = XMC_DMA_CH_TRANSFER_TYPE_MULTI_BLOCK_SRCADR_RELOAD_DSTADR_RELOAD, /* Transfer type */
  .priority = XMC_DMA_CH_PRIORITY_0, /* Priority level */
  .src_handshaking = XMC_DMA_CH_SRC_HANDSHAKING_HARDWARE, /* Source handshaking */
  .src_peripheral_request = DMA0_PERIPHERAL_REQUEST_USIC0_SR0_0, /* Source peripheral trigger */
  .dst_handshaking = XMC_DMA_CH_DST_HANDSHAKING_SOFTWARE  /* Destination handshaking */
};

static const XMC_UART_CH_CONFIG_t uart_config =
{
  .baudrate = 115200,
  .data_bits = 8,
  .stop_bits = 1,
};

static void uart_transmit(XMC_USIC_CH_t *const channel, const uint8_t *data, uint32_t len)
{
  for (uint32_t i = 0; i < len; ++i, ++data)
  {
	XMC_UART_CH_Transmit(channel, *data);
  }
}

void SysTick_Handler(void)
{
  static uint32_t start = 0;

  uint32_t end = XMC_DMA_CH_GetTransferredData(XMC_DMA0, 0);

  if (start != end)
  {
    if (start < end)
    {
      uart_transmit(XMC_UART0_CH0, (const uint8_t *)&ring_buffer[start], end - start); /* Process input data in linear buffer phase */
    }
    else
    {
      /*
       * In overflow mode we have to process twice:
       *  - Process data until end of buffer
       *  - Process data until current position on top of buffer
       */
      uart_transmit(XMC_UART0_CH0, (const uint8_t *)&ring_buffer[start], RING_BUFFER_SIZE - start); /* Process input data in linear buffer phase */
      uart_transmit(XMC_UART0_CH0, (const uint8_t *)&ring_buffer[0], end); /* Process input data in linear buffer phase */
    }

    start = end;
  }
}

int main(void)
{
  /* Enable DMA module */
  XMC_DMA_Init(XMC_DMA0);
  XMC_DMA_CH_Init(XMC_DMA0, 0, &dma_ch_config);
  XMC_DMA_CH_Enable(XMC_DMA0, 0);

  /* System timer configuration */
  SysTick_Config(SystemCoreClock / TICKS_PER_SECOND);

  /* UART configuration */
  XMC_UART_CH_Init(XMC_UART0_CH0, &uart_config);

  XMC_GPIO_SetMode(P1_4, XMC_GPIO_MODE_INPUT_TRISTATE);
  XMC_UART_CH_SetInputSource(XMC_UART0_CH0, XMC_UART_CH_INPUT_RXD, USIC0_C0_DX0_P1_4);

  /*Configure transmit FIFO*/
  XMC_USIC_CH_TXFIFO_Configure(XMC_UART0_CH0, 0U, XMC_USIC_CH_FIFO_SIZE_32WORDS, 0U);

  /* Set service request for receive interrupt */
  XMC_USIC_CH_SetInterruptNodePointer(XMC_UART0_CH0, XMC_USIC_CH_INTERRUPT_NODE_POINTER_RECEIVE, 0U);
  XMC_USIC_CH_SetInterruptNodePointer(XMC_UART0_CH0, XMC_USIC_CH_INTERRUPT_NODE_POINTER_ALTERNATE_RECEIVE, 0U);
  XMC_UART_CH_EnableEvent(XMC_UART0_CH0, XMC_UART_CH_EVENT_STANDARD_RECEIVE | XMC_UART_CH_EVENT_ALTERNATIVE_RECEIVE);

  XMC_UART_CH_Start(XMC_UART0_CH0);

  XMC_GPIO_SetMode(P1_5, (XMC_GPIO_MODE_t)((int32_t)XMC_GPIO_MODE_OUTPUT_PUSH_PULL | (int32_t)P1_5_AF_U0C0_DOUT0));

  /* Placeholder for user application code. The while loop below can be replaced with user application code. */
  while(1U);
}
